@@ -31,34 +31,32 @@
 #include "peng_ref.h"
 #include "sha2.h"
 #include "lpeng.h"
+#include "wolf64.h"
 
 
 /* global */ int verbosity = 0;
+/* global */ int debugmask = 0;
 
 static int init_done = 0;
 
 
-uint32_t eof_magic[] = { UINT32_C(0x1a68b01f), UINT32_C(0x4a11c153), UINT32_C(0x436621e9), UINT32_C(0xe710ffb4) };
-
-
 #define COMBINED_DIGEST_SIZE (WHIRLPOOL_DIGESTBYTES+SHA512_DIGEST_SIZE)
+#define PENG_MAGIC UINT32_C(0xec1f4a11)
+#define PENG_VER UINT16_C(0)
+#define PENG_CAP UINT16_C(0)
 
 
 void peng_unit_prep(void)
 {
-    int i;
-    
     if(init_done)
         return;
     init_done=1;
-    
-    for(i=0; i<(sizeof eof_magic/sizeof eof_magic[0]); i++)
-        eof_magic[i] = byte_reorder32(SYSTEM_BYTEORDER, TARGET_BYTEORDER, eof_magic[i], 4);
+    /* ... */
 }
 
 
 /* attention ! passphrase will be erased ! */
-void peng_cmd_prep(struct peng_cmd_environment *pce, uint32_t blksize, uint32_t rounds, uint32_t variations, char *passphrase, int eflag)
+void peng_cmd_prep(struct peng_cmd_environment *pce, uint64_t blksize, uint32_t rounds, uint32_t variations, char *passphrase, int eflag)
 {
     struct whirlpool wp;
     sha512_ctx sha512;
@@ -79,7 +77,7 @@ void peng_cmd_prep(struct peng_cmd_environment *pce, uint32_t blksize, uint32_t
     memcpy(combined, digest, WHIRLPOOL_DIGESTBYTES);
     memcpy(combined+WHIRLPOOL_DIGESTBYTES, digest2, SHA512_DIGEST_SIZE);
     
-    rectify(SYSTEM_BYTEORDER, TARGET_BYTEORDER, combined, COMBINED_DIGEST_SIZE);
+    rectify32(SYSTEM_BYTEORDER32, TARGET_BYTEORDER32, combined, COMBINED_DIGEST_SIZE);
     
     mersennetwister_init_by_array(&pce->mt, (uint32_t *)combined, COMBINED_DIGEST_SIZE/sizeof(uint32_t));  /* TODO byte order, packing */
     
@@ -98,14 +96,52 @@ void peng_cmd_prep(struct peng_cmd_environment *pce, uint32_t blksize, uint32_t
 }
 
 
+struct header
+  {
+      uint32_t        magic;
+      uint32_t        headerlen;
+      uint16_t        ver;
+      uint16_t        cap;
+      uint64_t        totalsize;
+      uint64_t        cksum;
+  };
+
 int peng_cmd_process(struct peng_cmd_environment *pce, const char *infn, int inh, uint64_t total, const char *outfn, int outh, char multithreading, char min_locrr_seq_len)
 {
-    int i,j,k,r,z;
-    int guess_eof = 0, truncate_at = 0;
-    uint32_t num=0, padding_remaining=0;
-    uint64_t pos=0;
+    int i,j,k,r;
+    int firstblock;
+    uint32_t num=0, off=0;
+    uint64_t pos=0, cksum;
+    struct header h;
+    struct header hwrit;
     
-    for(;;)
+    memset(&h, 0, sizeof h);  /* valgrind owed */
+    memset(&hwrit, 0, sizeof hwrit);
+    /* DEBUG_TIMING(1, "peng_cmd_process_0") */
+    
+    if(pce->eflag)
+    {
+        lseek(inh, 0, SEEK_SET);
+        h.cksum = wolf64(inh);
+        h.totalsize = lseek(inh, 0, SEEK_END);   /* it is already at the end... */
+        h.headerlen = sizeof h; 
+        h.magic = PENG_MAGIC;
+        h.cap = PENG_CAP;
+        h.ver = PENG_VER;
+        lseek(inh, 0, SEEK_SET);
+        
+        hwrit.headerlen = cvt_from_system64(h.headerlen);
+        hwrit.totalsize = cvt_from_system64(h.totalsize);
+        hwrit.cksum = cvt_from_system64(h.cksum);
+        hwrit.magic = cvt_from_system32(h.magic);
+        hwrit.cap = cvt_from_system16(h.cap);
+        hwrit.ver = cvt_from_system16(h.ver);
+        
+        memcpy(pce->buf1, &hwrit, sizeof hwrit);
+        off = sizeof h;
+    }
+
+    for(firstblock=1; ; firstblock=0)
     {
         if(verbosity>1)
         {
@@ -114,32 +150,22 @@ int peng_cmd_process(struct peng_cmd_environment *pce, const char *infn, int inh
         }
         
         /* memset(pce->buf1, 0, pce->bufsize); */
-        i = read(inh, pce->buf1, pce->bufsize);
+        i = read(inh, pce->buf1+off, pce->bufsize-off);
         if(i<0)
         {
             perror(infn);
             return -1;
         }
         
+        i += off;
         if(i<=0)
             break;
         
-        pos += i;
-        if(total>0 && pos>=total)
-            guess_eof = 1;
+        off = 0;
         
         if(pce->eflag && i<pce->bufsize)
         {
-            /* Pad buffer with random data and mark EOF.
-             * special case: the input file matches the block size exactly; then:
-             * do NOT use a EOF magic
-             */
-            padding_remaining = do_padding(pce->buf1+i, pce->bufsize-i, eof_magic, sizeof eof_magic / sizeof eof_magic[0], 0);
-        }
-        
-        if(!pce->eflag && i<pce->bufsize)
-        {
-            fprintf(stderr, "warning: %s: expected a full block while reading for decryption\n", outfn);
+            do_padding(pce->buf1+i, pce->bufsize-i);
         }
         
         /* memset(pce->buf2, 0, pce->bufsize); */
@@ -149,70 +175,67 @@ int peng_cmd_process(struct peng_cmd_environment *pce, const char *infn, int inh
         
         k = pce->eflag?(pce->bufsize):i;
         
-        if(!pce->eflag && guess_eof)
+        if(!pce->eflag && firstblock)
         {
-            /* Find the EOF marker.
-             */
-            z = locrr(pce->buf3, k, eof_magic, sizeof eof_magic / sizeof eof_magic[0], min_locrr_seq_len);
-            if(z>=-9999)   /* legal value */
+            /* read the header */
+            memcpy(&h, pce->buf3, sizeof h);
+            off = sizeof h;
+
+            h.ver = cvt_to_system16(h.ver);
+            h.cap = cvt_to_system16(h.cap);
+            h.magic = cvt_to_system32(h.magic);
+            h.headerlen = cvt_to_system32(h.headerlen);
+            h.totalsize = cvt_to_system64(h.totalsize);
+            h.cksum = cvt_to_system64(h.cksum);
+            if(h.magic!=PENG_MAGIC)
             {
-                if(verbosity>1)
-                    fprintf(stderr, "applying guess_eof at z=%d\n", z);
-                if(z>=0)
-                    k = z;
-                else
-                {
-                    /* z is negative and valid, that means the start of the marker was in the previous block */
-                    k = 0;
-                    truncate_at = z;
-                    break;
-                }
+                /* fprintf(stderr, "DEBUG: FAILED: ver=%"PRIx16" cap=%"PRIx16" magic=%"PRIx32"\n", h.ver, h.cap, h.magic); */
+                return 2;
+            }
+            if(h.ver>PENG_VER || h.cap!=PENG_CAP)
+            {
+                /* fprintf(stderr, "DEBUG: FAILED: ver=%"PRIx16" cap=%"PRIx16" magic=%"PRIx32"\n", h.ver, h.cap, h.magic);  */
+                return 3;
             }
         }
         
-        j = write(outh, pce->buf3, k);
+        j = write(outh, pce->buf3+off, k-off);
         if(j<0)
         {
             perror(outfn);
             return -1;
         }
-        if(k!=j)
+        if(k-off!=j)
         {
             fprintf(stderr, "warning: %s: bytes buffered not equal bytes written\n", outfn);
         }
-        if(guess_eof)
-            break;    /* this is to avoid some tricky problems (with growing files) */
+        off = 0;
     }
-    /* if(!pce->eflag && guess_eof && k==0 && z<0 && z>=-9999) */
-    if(truncate_at)
+
+    if(!pce->eflag)
     {
+        /* fprintf(stderr, "DEBUG: h.totalsize = %"PRIu64"\n", h.totalsize); */
         pos = lseek(outh, 0, SEEK_CUR);
         lseek(outh, 0, SEEK_SET);
-        r = ftruncate(outh, pos-truncate_at);
-        if(r)
+        if(pos>h.totalsize)
         {
-            perror(outfn);
-            return -1;
-        }
-    }
-    else
-    if(padding_remaining)  /* encrypting */
-    {
-        do_padding(pce->buf1, pce->bufsize, eof_magic, sizeof eof_magic / sizeof eof_magic[0], padding_remaining);
-        execpengpipe(pce->pp, pce->buf1, pce->buf2, pce->buf3, pce->eflag, multithreading);
-        k = pce->bufsize;
-        j = write(outh, pce->buf3, k);
-        if(j<0)
-        {
-            perror(outfn);
-            return -1;
-        }
-        if(k!=j)
-        {
-            fprintf(stderr, "warning: %s: bytes buffered not equal bytes written (fin)\n", outfn);
+            /* fprintf(stderr, "DEBUG: ftruncate() to %"PRIu64"\n", h.totalsize); */
+            r = ftruncate(outh, h.totalsize);
+            if(r)
+            {
+                perror(outfn);
+                return -1;
+            }
         }
+        lseek(outh, 0, SEEK_SET); /* some ftruncate()s work by re-positioning */
+        cksum = wolf64(outh);
+        /* fprintf(stderr, "DEBUG: %08lx (now) %"PRIx64" (stored)\n", cksum, h.cksum); */
+        if(cksum!=h.cksum)
+            return 1;
     }
     
+    /* DEBUG_TIMING(1, "peng_cmd_process_1") */
+    
     return 0;
 }
 
