@@ -41,13 +41,28 @@
 #define LICENSE "This program comes with ABSOLUTELY NO WARRANTY.\nLicensed under the GNU Public License version 3 or later.\n"
 
 
-const char *peng_version = "4.01.0070"; /* CHANGEME */
+const char *peng_version = "4.01.0076"; /* CHANGEME */
 
 
 #define MAXFNLEN              1024
 #define MIN_LOCRR_SEQ_LEN        8
 
 
+struct easyset
+{
+    int  cat;
+    char c;
+    int  v;
+} easysets[] =
+{
+    {0,'L',0x100}, {0,'M',0x1000}, {0,'H',0x10000}, {0,'X',0x100000}, 
+    {1,'L',5}, {1,'M',10}, {1,'H',20}, {1,'X',49}, 
+    {2,'L',1}, {2,'M',5}, {2,'H',11}, {2,'X',17},
+    
+    {-1,0,0}
+};
+
+
 void printversion(void)
 {
 #if ALPHA || BETA || DEBUG
@@ -74,7 +89,7 @@ void printversion(void)
 }
 
 
-/* returns array of numbers, first being the count */
+/* returns array of numbers, first item being the count */
 uint32_t *parseints(char *s)
 {
     int i0=0,i,j=1,n=1;
@@ -100,6 +115,29 @@ uint32_t *parseints(char *s)
 }
 
 
+/* returns array of numbers, first item being the count */
+uint32_t *parseeasy(char *s, struct easyset es[])
+{
+    int n=strlen(s), i;
+    int cat;
+    uint32_t *binparm = malloc((n+1)*sizeof(uint32_t));
+    
+    binparm[0]=n;
+    
+    for(cat=0; s[cat]; cat++)
+    {
+        binparm[cat+1]=0;
+        for(i=0; es[i].c; i++)
+            if(es[i].cat==cat && es[i].c==s[cat])
+            {
+                binparm[cat+1]=es[i].v;
+                break;
+            }
+    }
+    return binparm;
+}
+
+
 int main(int argc, char **argv)
 {
     int i,r,h1,h2;
@@ -110,7 +148,8 @@ int main(int argc, char **argv)
     int delflag;
     char fnmode = 'n';
     uint32_t *binparm;
-    uint32_t blksize, rounds, variations;
+    uint64_t blksize;
+    uint32_t rounds, variations;
     struct peng_cmd_environment mypce;
     char *origfn, infn[MAXFNLEN], outfn[MAXFNLEN], *passphrase=NULL;
     uint64_t total;
@@ -123,18 +162,21 @@ int main(int argc, char **argv)
     /* GNU: */
     /* + means: parse POSIXLY_CORRECT, stopping with the first non-option */
     /* - means: give opt==1 for any non-option parameter */
-    while((opt = getopt(argc, argv, "+hVO:drRnvP:m")) != -1)
+    while((opt = getopt(argc, argv, "+hVO:drRnvP:mD:")) != -1)
     {
         switch(opt) 
         {
             case 'O':
-                parm = strdup(optarg);  /* TODO leak */
+                parm = strdup(optarg);  /* TODO: leak */
                 if(!parm)
                 {
                     fputs("out of memory\n", stderr);
                     abort();
                 }
                 break;
+            case 'D':
+                debugmask = atoi(optarg);   /* you know what you are doing - ignore errors */
+                break;
             case 'r':
             case 'R':
             case 'n':
@@ -169,13 +211,15 @@ int main(int argc, char **argv)
                 fputs("options:\t-h\t\tthis help\n"
                       "\t\t-V\t\tdisplay version information\n"
                       "\t\t-O blocksize,rounds,variations\n"
+                      "\t\t-O [LMHX][LMHX][LMHX]\n"
                       "\t\t-d\t\tdecrypt\n"
                       "\t\t-r\t\trename input file to infile.bak\n"
                       "\t\t-R\t\treplace input file (dangerous!)\n"
                       "\t\t-n\t\tname output file as infile.{dec|enc} (default)\n"
                       "\t\t-v\t\tincrease verbosity\n"
                       "\t\t-P passphrase\n"
-                      "\t\t-m\t\tenable multithreading\n", stdout);
+                      "\t\t-m\t\tenable multithreading\n"
+                      "\t\t-D mask\t\tset debugging\n", stdout);
                 return 1;
         }
     }
@@ -185,11 +229,23 @@ int main(int argc, char **argv)
         return 9;
     }
     
-    binparm = parseints(parm);
-    
-    if(binparm[0]!=3)
+    if(strlen(parm)==3)    /* EASY */
     {
-        fprintf(stderr, "expected -O parameter options as triple numeric separated by commas\n");
+        binparm = parseeasy(parm, easysets);
+    }
+    else
+    {
+        binparm = parseints(parm);
+    }
+    
+    r=0;
+    for(i=1; i<binparm[0]; i++)
+        if(binparm[i]<1)
+            r=1;
+    
+    if(binparm[0]!=3 || r)
+    {
+        fprintf(stderr, "expected -O parameter as triple numeric separated by commas\nor three letters out of [LMHX]\n");
         return 9;
     }
     blksize = binparm[1];
@@ -197,6 +253,11 @@ int main(int argc, char **argv)
     variations = binparm[3];
     FREE(binparm);
     
+    if(verbosity>0)
+    {
+        printf("blksize=%"PRIu64" variations=%"PRIu32" rounds=%"PRIu32"\n", blksize, variations, rounds);
+    }
+    
     if(!passphrase)
         passphrase = getpass("PENG Password: ");
     
@@ -254,7 +315,7 @@ int main(int argc, char **argv)
             perror(infn);
             return -1;
         }
-        h2 = open(outfn, O_WRONLY|O_CREAT|O_TRUNC, 0600);
+        h2 = open(outfn, O_RDWR|O_CREAT|O_TRUNC, 0600);
         if(h2<0)
         {
             perror(outfn);
@@ -275,6 +336,22 @@ int main(int argc, char **argv)
         close(h1);
         close(h2);
         
+        if(r>0)  /* checksum error */
+        {
+            switch(r)
+            {
+                case 1:
+                    fprintf(stderr, "%s: checksum mismatch\n", argv[i]);
+                    break;
+                case 2:
+                    fprintf(stderr, "%s: magic missing (password wrong?)\n", argv[i]);
+                    break;
+                case 3:
+                    fprintf(stderr, "%s: compatibility (version or capabilities) failure\n", argv[i]);
+                    break;
+            }
+            return 10;
+        }
         if(r<0)
         {
             /* perror(argv[i]); <<< this is done in the process */
