@@ -66,7 +66,7 @@ static void qbitcopy(const uint8_t *buf1, uint32_t off1, uint32_t max1, uint8_t
 {
     if(off1/8>=max1 || off2/8>=max2)
     {
-        fprintf(stderr, "PANIC: %u>%u or %u>%u\n", off1/8, max1, off2/8, max2);
+        fprintf(stderr, "PANIC: %u>=%u or %u>=%u\n", off1/8, max1, off2/8, max2);
         abort();
     }
     
@@ -80,35 +80,37 @@ static void qbitcopy(const uint8_t *buf1, uint32_t off1, uint32_t max1, uint8_t
 #endif
 
 
-struct pengset *genpengset(uint32_t blksize, struct mersennetwister *mt)
+struct pengset *genpengset(uint64_t blksize, struct mersennetwister *mt)
 {
-    uint32_t blksize8 = blksize*8;
+    uint64_t blksize8 = blksize*8;
     struct pengset *res = MALLOC(sizeof(struct pengset));
     char *tempflg1 = MALLOCA(blksize8*sizeof(char));
     char *tempflg2 = MALLOCA(blksize8*sizeof(char));
     int i,j,k;
     
-    memset(tempflg1, 0, blksize8);
-    memset(tempflg2, 0, blksize8);
+    memset(tempflg1, 0, blksize8*sizeof(char));
+    memset(tempflg2, 0, blksize8*sizeof(char));
     res->blksize = blksize;
-    res->perm1 = MALLOC(blksize8*sizeof(uint16_t));
-    res->perm2 = MALLOC(blksize8*sizeof(uint16_t));
+    res->perm1 = MALLOC(blksize8*sizeof(uint32_t));
+    res->perm2 = MALLOC(blksize8*sizeof(uint32_t));
     res->mask1  = MALLOC(blksize*sizeof(uint8_t));
 #if USE_MODE_XPX
     res->mask2  = MALLOC(blksize*sizeof(uint8_t));
 #endif
-    memset(res->perm1, 0, blksize8);
-    memset(res->perm2, 0, blksize8);
+    memset(res->perm1, 0, blksize8*sizeof(uint32_t));
+    memset(res->perm2, 0, blksize8*sizeof(uint32_t));
+    
+    /* DEBUG_TIMING(1, "genpengset_0") */
     
     for(i=0; i<blksize8; i++)
     {
 #if DORKINESS>1
         j = mersennetwister_genrand_int32(mt) % blksize8;
-        /* this is dorky, but it speeds things up a lot */
+        /* this is dorky, but it speeds things up a lot (?) */
         while(tempflg1[j])
             j = (j+1)%blksize8;
         k = mersennetwister_genrand_int32(mt) % blksize8;
-        /* this is dorky, but it speeds things up a lot */
+        /* this is dorky, but it speeds things up a lot (?) */
         while(tempflg2[k])
             k = (k+1)%blksize8;
 #elif DORKINESS==1
@@ -144,11 +146,13 @@ struct pengset *genpengset(uint32_t blksize, struct mersennetwister *mt)
 #endif
     }
     
+    /* DEBUG_TIMING(1, "genpengset_1") */
+    
     return res;
 }
 
 
-struct pengpipe *genpengpipe(uint32_t blksize, uint32_t rounds, uint32_t variations, struct mersennetwister *mt)
+struct pengpipe *genpengpipe(uint64_t blksize, uint32_t rounds, uint32_t variations, struct mersennetwister *mt)
 {
     int i,j;
     struct pengpipe *res = MALLOC(sizeof(struct pengpipe));
@@ -167,7 +171,7 @@ struct pengpipe *genpengpipe(uint32_t blksize, uint32_t rounds, uint32_t variati
         {
             if(verbosity>1)
             {
-                printf("generating variation=%d, round=%d  \r", i, j);
+                printf("generating variation=%d, round=%d  \r", i+1, j+1);
                 fflush(stdout);
             }
             res->mtx[i][j] = genpengset(blksize, mt);
@@ -222,8 +226,8 @@ void destroypengpipe(struct pengpipe *p)
 
 void execpengset(struct pengset *p, const uint8_t *buf1, uint8_t *tmpbuf, uint8_t *buf2, char encrypt)
 {
-    uint32_t blksize = p->blksize;
-    uint32_t blksize8 = blksize*8;
+    uint64_t blksize = p->blksize;
+    uint64_t blksize8 = blksize*8;
     int i;
     
     if(encrypt)
@@ -269,19 +273,34 @@ void execpengset(struct pengset *p, const uint8_t *buf1, uint8_t *tmpbuf, uint8_
 
 struct epp_thr_context
 {
-    struct pengset    **mtx;
-    uint32_t            rounds;
-    uint32_t            blksize;
-    uint8_t      *buf1;
-    uint8_t      *tmpbuf;
-    uint8_t      *buf2;
+    struct pengset        **mtx;
+    uint32_t                rounds;
+    uint64_t                blksize;
+    uint8_t                *buf1;
+    uint8_t                *tmpbuf;
+    uint8_t                *buf2;
     char encrypt;
 #if USE_MODE_CBC
-    uint8_t      *iv;
+    uint8_t                *iv;
 #endif
 };
 
 
+/**************************************
+struct rnd_thr_context
+{
+    struct mersennetwister *mt;
+    int                     wrh;
+};
+
+
+static void *rnd_thr(void *param)
+{
+    return NULL;
+}
+**************************************/
+
+
 static void *epp_thr(void *param)
 {
     int j;
@@ -396,7 +415,7 @@ void execpengpipe(struct pengpipe *p, uint8_t *buf1, uint8_t *tmpbuf, uint8_t *b
 }
 
 
-uint32_t getbufsize(struct pengpipe *p)
+uint64_t getbufsize(struct pengpipe *p)
 {
-    return (uint32_t)p->blksize * p->variations;
+    return (uint64_t)p->blksize * p->variations;
 }
